package exnihilocreatio.api.registries

import exnihilocreatio.items.ore.ItemOre
import exnihilocreatio.items.ore.Ore
import exnihilocreatio.texturing.Color
import exnihilocreatio.util.ItemInfo

interface IOreRegistry : IRegistryList<Ore> {
    val itemOreRegistry: List<ItemOre>

    /**
     * A black list of ores to not register
     */
    val sieveBlackList: Set<ItemOre>

    /**
     * Registers a new custom piece, hunk, dust and potentially ingot to be
     * generated by Ex Nihilo Creatio.
     *
     * @param name  Unique name for ore
     * @param color Color for the pieces
     * @param info  Final result for the process. If null, an ingot is generated.
     * Otherwise, the hunk will be smelted into this.
     * @return Ore, containing the base Ore object.
     */
    fun register(name: String, color: Color, info: ItemInfo? = null, translations: Map<String, String>? = null, oredictName: String? = null): Ore

    fun getOreItem(name: String): ItemOre?
    fun isRegistered(name: String): Boolean
}